import bpy
from render_nox.env.sunsky_ops import eval_sun_position

#-----------------------------------

class NOXSunskyPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "sunskypanel"
    bl_label = "NOX Sunsky"
    bl_context = "world"
    COMPAT_ENGINES = {"nox_renderer"}

    def draw_header(self, context):
        world = context.scene.world
        if world:
            if hasattr(world, "noxData"):
                self.layout.prop(world.noxData, "active", text="")

    def draw(self, context):
        row = self.layout.row()
        row.template_ID(context.scene, "world", new="world.new")
        self.layout.separator()

        world = context.scene.world
        
        if world:
            if hasattr(world, "noxData"):
            
                esp = eval_sun_position(world)
                if esp[0]:
                    sun_altitude = esp[1]
                    sun_azimuth = esp[2]
                else:
                    sun_altitude = world.noxData.sun_altitude
                    sun_azimuth = world.noxData.sun_azimuth
                    
                row = self.layout.row()
                row.prop(world.noxData, "active")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Time")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "month")
                row.prop(world.noxData, "day")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "hour")
                row.prop(world.noxData, "minute")
                row = self.layout.row()
                row.prop(world.noxData, "gmt")
                self.layout.separator()
                self.layout.separator()


                row = self.layout.row()
                row.label(text="Scene position")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "longitude")
                row.prop(world.noxData, "latitude")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Orientation")
                row = self.layout.row()
                row.prop(world.noxData, "north_angle")
                self.layout.separator()
                self.layout.separator()
                

                row = self.layout.row()
                row.label(text="Sun")
                row = self.layout.row()
                row.prop(world.noxData, "sun_on")
                row = self.layout.row()
                row.prop(world.noxData, "sun_layer2")
                row = self.layout.row()
                row.prop(world.noxData, "sun_size")
                row = self.layout.row()
                row.label(text="Altitude: " + str(int(sun_altitude)))
                #row = self.layout.row()
                row.label(text="Azimuth: " + str(int(sun_azimuth)))
                #row.operator("world.eval_sun_pos", text="Refresh")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Atmosphere")
                row = self.layout.row()
                row.prop(world.noxData, "turbidity")
                




                
    @classmethod
    def poll(cls, context):
        return (context.scene.render.engine == 'nox_renderer')

#-----------------------------------
