import bpy

#-----------------------------------

class NOXSunskyProps(bpy.types.PropertyGroup):
    active = bpy.props.BoolProperty(
        name = "Sunsky active", 
        description = "Use sunsky lighting on this scene", 
        default = False
    )
    month = bpy.props.IntProperty(
        name = "Month",
        description = "",
        default = 1,
        min = 1,
        max = 12
    )
    day = bpy.props.IntProperty(
        name = "Day",
        description = "",
        default = 1,
        min = 1,
        max = 31
    )
    hour = bpy.props.IntProperty(
        name = "Hour",
        description = "",
        default = 12,
        min = 0,
        max = 23
    )
    minute = bpy.props.IntProperty(
        name = "Minute",
        description = "",
        default = 0,
        min = 0,
        max = 59
    )
    gmt = bpy.props.IntProperty(
        name = "GMT",
        description = "Timezone shift from GMT",
        default = 0,
        min = -12,
        max = 12
    )
    
    longitude = bpy.props.IntProperty(
        name = "Longitude",
        description = "Longitude position - positive for east, negative for west",
        default = 0,
        min = -180,
        max = 179
    )
    latitude = bpy.props.IntProperty(
        name = "Latitude",
        description = "Latitude position - positive for south, negative for north",
        default = 0,
        min = -89,
        max = 89
    )
    
    sun_on = bpy.props.BoolProperty(
        name = "Sun active", 
        description = "Eval sun illumination", 
        default = True
    )
    sun_layer2 = bpy.props.BoolProperty(
        name = "Sun on blend layer 2", 
        description = "Send sun illumination to 2nd blend layer. Otherwise on 1st layer as the sky.", 
        default = False
    )
    sun_size = bpy.props.FloatProperty(
        name = "Sun size",
        description = "Angular sun size - default 0.533 degrees as on Earth",
        default = 0.5333333,
        min = 0.1,
        max = 2.0
    )
    
    sun_altitude = bpy.props.FloatProperty(
        name = "Altitude",
        description = "Sun altitude in degrees - needs to be manually refreshed",
        default = 0.0,
        min = -90.0,
        max = 90.0
    )
    sun_azimuth = bpy.props.FloatProperty(
        name = "Azimuth",
        description = "Sun azimuth in degrees - needs to be manually refreshed. 0 - N, 90 - E, 180 - S, 270 - W",
        default = 0.0,
        min = -90.0,
        max = 90.0
    )
    turbidity = bpy.props.IntProperty(
        name = "Turbidity",
        description = "Sky turbidity",
        default = 30,
        min = 20,
        max = 100
    )

    north_angle = bpy.props.FloatProperty(
        name = "North direction angle",
        description = "North direction angle. 0 for Y, 90 for X, 180 for -Y, 270 for -X",
        default = 0.0,
        min = 0.0,
        max = 360.0
    )
    
#-----------------------------------
