import bpy
from mathutils import Color
from xml.dom import minidom
import struct
#from io_utils import ImportHelper
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty, BoolProperty, EnumProperty


nox_xml_error_line = -1
nox_xml_error_node = ""

#----------------------------------------------------------------

def import_material(filename, mnum):
    global nox_xml_error_node
    if mnum<0 or mnum >= bpy.data.materials.__len__():
        return False
    
    mat = bpy.data.materials[mnum]
    if not hasattr(mat, "noxData"):
        return False

    xmlTree = minidom.parse(filename)

    rootNodes = xmlTree.childNodes
    if rootNodes.__len__() != 1:
        return False
    
    xmlRoot = rootNodes[0]
    if xmlRoot.nodeName!="material":
        nox_xml_error_node = xmlBlend.nodeName
        return False
    
    # <blendlayer>
    for xmlBlend in xmlRoot.getElementsByTagName("blendlayer"):
        res = parseForInt(xmlBlend)
        if not res[0]:
            return False
        if res[1]<1 or res[1]>16:
            nox_xml_error_node = xmlBlend.nodeName
            return False
        mat.noxData.blend_layer = "b"+str(res[1])
        
    # <skyportal> 
    for xmlSkyportal in xmlRoot.getElementsByTagName("skyportal"):
        if not xmlSkyportal.childNodes:
            nox_xml_error_node = xmlSkyportal.nodeName
            return False
        if xmlSkyportal.childNodes[0].nodeType != xmlSkyportal.TEXT_NODE:
            nox_xml_error_node = xmlSkyportal.nodeName
            return False
        if xmlSkyportal.childNodes[0].data == "yes":
            mat.noxData.skyportal = True
        else:
            mat.noxData.skyportal = False

    # <opacity>
    for xmlOpac in xmlRoot.getElementsByTagName("opacity"):
        res = parseForFloat(xmlOpac)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlOpac.nodeName
            return False
        mat.noxData.opacity = res[1]

    # del old layers
    ll = mat.noxData.layers.__len__()
    for i in range(ll):
        mat.noxData.layers.remove(ll-i-1)

    # <layer>
    for xmlmatlayer in xmlRoot.getElementsByTagName("layer"):
        res = parseMatLayer(xmlmatlayer, mat)
        if not res:
            return False

    # <tex_opacity>
    for xmlTex in xmlRoot.getElementsByTagName("tex_opacity"):
        res = parseTexture(xmlTex, mat, 0)
        if not res:
            return False
            
    # <preview>
    for xmlpreview in xmlRoot.getElementsByTagName("preview"):
        res = parsePreview(xmlpreview, mat)
        if not res:
            return False
    
    return True

#----------------------------------------------------------------

def parseTexture(xmlTex, mat, cLay):
    if xmlTex.nodeName=='tex_opacity':
        tslot = mat.noxData.tex_opacity
    else:
        if cLay>= mat.noxData.layers.__len__() or cLay<0:
            nox_xml_error_node = xmlTex.nodeName
            return False
        layer = mat.noxData.layers[cLay]
        if xmlTex.nodeName=='tex_refl0':
            tslot = layer.tex_refl0
        if xmlTex.nodeName=='tex_refl90':
            tslot = layer.tex_refl90
        if xmlTex.nodeName=='tex_rough':
            tslot = layer.tex_rough
        if xmlTex.nodeName=='tex_weight':
            tslot = layer.tex_weight
        if xmlTex.nodeName=='tex_light':
            tslot = layer.tex_light
        if xmlTex.nodeName=='tex_normal':
            tslot = layer.tex_normal
    
    attr_enabled = xmlTex.getAttribute("enabled")
    if attr_enabled == "yes":
        tslot.enabled = True
    else:
        tslot.enabled = False
            
    # <filter>
    for xmlFilter in xmlTex.getElementsByTagName("filter"):
        res = parseForInt(xmlFilter)
        if not res[0]:
            return False
        if res[1]==1:
            tslot.bilinear= True
        else:
            tslot.bilinear = False

    # <filename>
    for xmlFilename in xmlTex.getElementsByTagName("filename"):
        if not xmlFilename.childNodes:
            nox_xml_error_node = xmlFilename.nodeName
            return (False, 0)
        if xmlFilename.childNodes[0].nodeType != xmlFilename.TEXT_NODE:
            nox_xml_error_node = xmlFilename.nodeName
            return (False, 0)
        if xmlFilename.childNodes[0].data != "":
            res = findOrLoadTexture(xmlFilename.childNodes[0].data)
            if res>=0:
                tslot.texID = res

    # <post>
    for xmlPost in xmlTex.getElementsByTagName("post"):

        # <brightness>
        for xmlBrightness in xmlPost.getElementsByTagName("brightness"):
            res = parseForFloat(xmlBrightness)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlBrightness.nodeName
                return False
            tslot.brightness = res[1]
    
        # <contrast>
        for xmlContrast in xmlPost.getElementsByTagName("contrast"):
            res = parseForFloat(xmlContrast)
            if not res[0]:
                return False
            if res[1]<0.0 or res[1]>4.0:
                nox_xml_error_node = xmlContrast.nodeName
                return False
            tslot.contrast = res[1]

        # <saturation>
        for xmlSaturation in xmlPost.getElementsByTagName("saturation"):
            res = parseForFloat(xmlSaturation)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlSaturation.nodeName
                return False
            tslot.saturation = res[1]

        # <hue>
        for xmlHue in xmlPost.getElementsByTagName("hue"):
            res = parseForFloat(xmlHue)
            if not res[0]:
                return False
            if res[1]<0.0 or res[1]>4.0:
                nox_xml_error_node = xmlHue.nodeName
                return False
            tslot.hue = res[1]

        # <red>
        for xmlRed in xmlPost.getElementsByTagName("red"):
            res = parseForFloat(xmlRed)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlRed.nodeName
                return False
            tslot.red = res[1]

        # <green>
        for xmlGreen in xmlPost.getElementsByTagName("green"):
            res = parseForFloat(xmlGreen)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlGreen.nodeName
                return False
            tslot.green = res[1]

        # <blue>
        for xmlBlue in xmlPost.getElementsByTagName("blue"):
            res = parseForFloat(xmlBlue)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlBlue.nodeName
                return False
            tslot.blue = res[1]

        # <gamma>
        for xmlGamma in xmlPost.getElementsByTagName("gamma"):
            res = parseForFloat(xmlGamma)
            if not res[0]:
                return False
            if res[1]<1.0 or res[1]>4.0:
                nox_xml_error_node = xmlGamma.nodeName
                return False
            tslot.gamma = res[1]

        # <invert>
        for xmlInvert in xmlPost.getElementsByTagName("invert"):
            if not xmlInvert.childNodes:
                nox_xml_error_node = xmlInvert.nodeName
                return (False, 0)
            if xmlInvert.childNodes[0].nodeType != xmlInvert.TEXT_NODE:
                nox_xml_error_node = xmlInvert.nodeName
                return (False, 0)
            if xmlInvert.childNodes[0].data == "yes":
                tslot.invert = True
            else:
                tslot.invert = False

    return True
    
#----------------------------------------------------------------

def parseMatLayer(xmlLayer, mat):
    global nox_xml_error_node
    
    if mat.noxData.layers.__len__()>15:
        nox_xml_error_node = xmlLayer.nodeName + " too many!"
        return False
    
    mat.noxData.layers.add()
    cLay = mat.noxData.layers.__len__()-1
    mlay = mat.noxData.layers[cLay]
    mlay.name = "Layer " + str(mat.noxData.layers.__len__())
    
    # <layer type>
    ltype = xmlLayer.getAttribute("type")
    if ltype=="shade":
        mlay.emission = False
    elif ltype=="emission":
        mlay.emission = True
    else:
        nox_xml_error_node = xmlLayer.nodeName
        return False
        
    # <layer contribution>
    scontr = xmlLayer.getAttribute("contribution")
    try:
        v = float(scontr)
    except ValueError:
        nox_xml_error_node = xmlLayer.nodeName
        return False
    mlay.weight = v
    
    # <diffuse>
    for xmlDiff in xmlLayer.getElementsByTagName("diffuse"):
        res = parseForColorHex(xmlDiff)
        if not res[0]:
            return False
        mlay.refl0.r = float(res[1])/255.0
        mlay.refl0.g = float(res[2])/255.0
        mlay.refl0.b = float(res[3])/255.0
    
    # <diffuse90>
    for xmlDiff in xmlLayer.getElementsByTagName("diffuse90"):
        res = parseForColorHex(xmlDiff)
        if not res[0]:
            return False
        mlay.refl90.r = float(res[1])/255.0
        mlay.refl90.g = float(res[2])/255.0
        mlay.refl90.b = float(res[3])/255.0

    # <roughness>
    for xmlRough in xmlLayer.getElementsByTagName("roughness"):
        res = parseForFloat(xmlRough)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlRough.nodeName
            return False
        mlay.rough = res[1]

    # <fresnel>
    for xmlFres in xmlLayer.getElementsByTagName("fresnel"):
        # <ior>
        for xmlIOR in xmlFres.getElementsByTagName("ior"):
            res = parseForFloat(xmlIOR)
            if not res[0]:
                return False
            if res[1]<1.0 or res[1]>100.0:
                nox_xml_error_node = xmlIOR.nodeName
                return False
            mlay.ior = res[1]
                
    # <transmission>
    for xmlTransm in xmlLayer.getElementsByTagName("transmission"):
        # <transmission active>
        tract = xmlTransm.getAttribute("active")
        if tract=="yes":
            mlay.transmission = True
        else:
            mlay.transmission = False
           
        # <fakeglass> 
        for xmlFakeglass in xmlTransm.getElementsByTagName("fakeglass"):
            if not xmlFakeglass.childNodes:
                nox_xml_error_node = xmlFakeglass.nodeName
                return (False, 0)
            if xmlFakeglass.childNodes[0].nodeType != xmlFakeglass.TEXT_NODE:
                nox_xml_error_node = xmlFakeglass.nodeName
                return (False, 0)
            if xmlFakeglass.childNodes[0].data == "yes":
                mlay.fake_glass = True
            else:
                mlay.fake_glass = False

        #<absorption>        
        for xmlAbsorpt in xmlTransm.getElementsByTagName("absorption"):
            # <absorption active>
            tract = xmlAbsorpt.getAttribute("active")
            if tract=="yes":
                mlay.absorption = True
            else:
                mlay.absorption = False
    
            # <abscolor>
            for xmlAbsCol in xmlAbsorpt.getElementsByTagName("abscolor"):
                res = parseForColorHex(xmlAbsCol)
                if not res[0]:
                    return False
                mlay.absorption_color.r = float(res[1])/255.0
                mlay.absorption_color.g = float(res[2])/255.0
                mlay.absorption_color.b = float(res[3])/255.0
            
            # <distance>
            for xmlAbsDist in xmlAbsorpt.getElementsByTagName("distance"):
                res = parseForFloat(xmlAbsDist)
                if not res[0]:
                    return False
                if res[1]<0.0 or res[1]>100.0:
                    nox_xml_error_node = xmlAbsDist.nodeName
                    return False
                mlay.absorption_distance = res[1]
        
    # <emission>
    for xmlEmissCol in xmlLayer.getElementsByTagName("emission"):
        res = parseForColorHex(xmlEmissCol)
        if not res[0]:
            return False
        mlay.emission_color.r = float(res[1])/255.0
        mlay.emission_color.g = float(res[2])/255.0
        mlay.emission_color.b = float(res[3])/255.0
    
    # <power>
    for xmlEmissPow in xmlLayer.getElementsByTagName("power"):
        res = parseForFloatPackedHex(xmlEmissPow)
        if not res[0]:
            return False
        if res[1]<0.0:
            nox_xml_error_node = xmlEmissPow.nodeName
            return False
        mlay.emission_power = res[1]
       
    # <unit>
    for xmlUnit in xmlLayer.getElementsByTagName("unit"):
        if not xmlUnit.childNodes:
            nox_xml_error_node = xmlUnit.nodeName
            return (False, 0)
        if xmlUnit.childNodes[0].nodeType != xmlUnit.TEXT_NODE:
            nox_xml_error_node = xmlUnit.nodeName
            return (False, 0)
        ustr = xmlUnit.childNodes[0].data
        mlay.emission_unit = 'W'
        if ustr=="w":
            mlay.emission_unit = 'W'
        if ustr=="w/m2":
            mlay.emission_unit = 'Wm'
        if ustr=="lm":
            mlay.emission_unit = 'lm'
        if ustr=="lx":
            mlay.emission_unit = 'lx'
        
    # <temperature>
    for xmlTemperature in xmlLayer.getElementsByTagName("temperature"):
        res = parseForInt(xmlTemperature)
        if not res[0]:
            return False
        if res[1]<1000 or res[1]>10000:
            nox_xml_error_node = xmlTemperature.nodeName
            return False
        mlay.emission_temperature = res[1]

    # <tex_refl0>
    for xmlTex in xmlLayer.getElementsByTagName("tex_refl0"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_refl90>
    for xmlTex in xmlLayer.getElementsByTagName("tex_refl90"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_rough>
    for xmlTex in xmlLayer.getElementsByTagName("tex_rough"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_weight>
    for xmlTex in xmlLayer.getElementsByTagName("tex_weight"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_light>
    for xmlTex in xmlLayer.getElementsByTagName("tex_light"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False
            
    # <tex_normal>
    for xmlTex in xmlLayer.getElementsByTagName("tex_normal"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    return True

#----------------------------------------------------------------
    
def parseForInt(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    try:
        v = int(xmlNode.childNodes[0].data)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    return (True, v)

#----------------------------------------------------------------
    
def parseForFloat(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    try:
        v = float(xmlNode.childNodes[0].data)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    return (True, v)

#----------------------------------------------------------------

def parseForColorHex(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    hexstr = xmlNode.childNodes[0].data
    if len(hexstr) != 6:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    hr = "{0}{1}".format(*hexstr)
    hg = "{2}{3}".format(*hexstr)
    hb = "{4}{5}".format(*hexstr)
    try:
        dr = int(hr, 16)
        dg = int(hg, 16)
        db = int(hb, 16)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    
    return (True, dr,dg,db)

#----------------------------------------------------------------

def parseForFloatPackedHex(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    hexstr = xmlNode.childNodes[0].data
    if len(hexstr) != 8:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    h1 = "{1}{0}".format(*hexstr)
    h2 = "{3}{2}".format(*hexstr)
    h3 = "{5}{4}".format(*hexstr)
    h4 = "{7}{6}".format(*hexstr)
    try:
        d1 = int(h1, 16)
        d2 = int(h2, 16)
        d3 = int(h3, 16)
        d4 = int(h4, 16)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    
    aastr = struct.pack("=BBBB", d1,d2,d3,d4)
    f = struct.unpack("=f", aastr)
    
    return (True, f[0])

#----------------------------------------------------------------

def parsePreview(xmlPreview, mat):
    global nox_xml_error_node
    
    #return True

    strwidth  = xmlPreview.getAttribute("width")
    strheight = xmlPreview.getAttribute("height")
    print("width = "+strwidth)
    print("height = "+strheight)
    try:
        iw = int(strwidth)
        ih = int(strheight)
    except ValueError:
        nox_xml_error_node = xmlPreview.nodeName
        return False
    
    #if mat.noxData.preview_id<0 or mat.noxData.preview_id>=bpy.data.images.__len__():
    #    bpy.ops.image.new(width=iw, height=ih)
    #    mat.noxData.preview_id = bpy.data.images.__len__()-1
    #img = bpy.data.images[mat.noxData.preview_id]
    #bpy.ops.texture.new()
    #tid = bpy.data.textures.__len__()-1
    #bpy.data.textures[tid].type="IMAGE"
    #bpy.data.textures[tid].type_recast


    if mat.noxData.preview_id<0 or mat.noxData.preview_id>=bpy.data.images.__len__():
        bpy.ops.image.new(width=iw, height=ih)
        mat.noxData.preview_id = bpy.data.images.__len__()-1
    img = bpy.data.images[mat.noxData.preview_id]

    
    
    
    
    arr = []
    for i in range(ih):
        nodestr = "l"+str(i)
        # <lxxx>
        lstart = iw*4*i
        #print(str(lstart))
        for xmlLine in xmlPreview.getElementsByTagName(nodestr):
            if not xmlLine.childNodes:
                nox_xml_error_node = xmlLine.nodeName
                return False
            if xmlLine.childNodes[0].nodeType != xmlLine.TEXT_NODE:
                nox_xml_error_node = xmlLine.nodeName
                return False
            ustr = xmlLine.childNodes[0].data
            #print(nodestr)
            if len(ustr) != iw * 6:
                nox_xml_error_node = xmlLine.nodeName
                return False
            for j in range(iw):
                dst = lstart + j*4
                src = j*6
            #    srcformat = "{"+str(src+0)+"}"+"{"+str(src+1)+"}"+"{"+str(src+2)+"}"+"{"+str(src+3)+"}"+"{"+str(src+4)+"}"+"{"+str(src+5)+"}"
            #    substring = srcformat.format(*ustr)
            #    hr = "{0}{1}".format(*substring)
            #    hg = "{2}{3}".format(*substring)
            #    hb = "{4}{5}".format(*substring)
                
                #srcfmt1 = "{"+str(src+0)+"}"+"{"+str(src+1)+"}"
                #srcfmt2 = "{"+str(src+2)+"}"+"{"+str(src+3)+"}"
                #srcfmt3 = "{"+str(src+4)+"}"+"{"+str(src+5)+"}"
                #hr = srcfmt1.format(*ustr)
                #hg = srcfmt2.format(*ustr)
                #hb = srcfmt3.format(*ustr)
                
                hr =  ustr[src+0]+ustr[src+1]
                hg =  ustr[src+2]+ustr[src+3]
                hb =  ustr[src+4]+ustr[src+5]
                
                try:
                    dr = int(hr, 16)
                    dg = int(hg, 16)
                    db = int(hb, 16)
                except ValueError:
                    nox_xml_error_node = xmlNode.nodeName
                    return False
                fr = float(dr)/255.0
                fg = float(dg)/255.0
                fb = float(db)/255.0
                arr.append(fr)
                arr.append(fg)
                arr.append(fb)
                arr.append(1.0)
                #img.pixels[dst+0] = fr
                #img.pixels[dst+1] = fg
                #img.pixels[dst+2] = fb
                #img.pixels[dst+3] = 1.0

    img.pixels = arr
    return True    

#----------------------------------------------------------------

def findOrLoadTexture(filename):
    res = -1
    print("trying "+filename)
    for i in range(bpy.data.textures.__len__()):
        tex = bpy.data.textures[i]   
        if hasattr(tex, "image") and tex.type=="IMAGE":
            if hasattr(tex.image, 'filepath') and hasattr(tex.image, 'source'):
                if tex.image.source=='FILE' and tex.image.filepath==filename:
                    print("already loaded id: "+str(i))
                    res = i
    if res==-1:
        try:
            bpy.ops.image.open(filepath=filename)
        except RuntimeError:
            return -1

        imgid = -1
        for i in range(bpy.data.images.__len__()):
            timg = bpy.data.images[i]
            if timg.type=='IMAGE' and timg.filepath==filename:
                imgid = i

        if imgid<0:
            print("Internal error, loaded texture but can't find it. Funny, I know...")
            return -1
            
        #imgid = bpy.data.images.__len__()-1
        img = bpy.data.images[imgid]

        print("loaded img id: "+str(imgid))

        bpy.ops.texture.new()
        texID = bpy.data.textures.__len__()-1
        tex = bpy.data.textures[texID]
        tex.type = 'IMAGE'
        tex.type_recast()
        tex = bpy.data.textures[texID]
        tex.image = img
        res = texID

        print("and tex id: "+str(texID))
        
    return res
    
#----------------------------------------------------------------

class NOXMaterialImporter(bpy.types.Operator, ImportHelper):
    bl_idname = "wm.buttonimportnoxmaterial"
    bl_label = "Import NOX Material"
    filename_ext = ".nxm"
    filter_glob = StringProperty(default="*.nxm", options={'HIDDEN'})
    	
    #addTex = BoolProperty(name="Copy textures", 
    #    description="Copy used textures - not implemented", default=False)
       
    def invoke(self, context, event):
        wm = context.window_manager
        wm.fileselect_add(self)
        return {"RUNNING_MODAL"}
        
    def execute(self, context):
        matID = -1
        matname = ""
        if hasattr(context, "material"):
            if hasattr(context.material, "name"):
                matname = context.material.name
                nummats = bpy.data.materials.__len__()
                for i in range(nummats):
                    if matname == bpy.data.materials[i].name:
                        matID = i
						
        if matID > -1:
            import_material(self.filepath, matID)
        return {"FINISHED"}
    
def menu_func_import(self, context):
    self.layout.operator(NOXMaterialImporter.bl_idname, text="NOX Material Import (*.nxm)")

#----------------------------------------------------------------
    
