import bpy
import tempfile
from render_nox.material.material_ops import runNOX, getNOXpath
from render_nox.export.export_nox_scene import write_NOX_data
import time

#---------------------------------------------------------------------------------------------------

class NOXRenderEngine(bpy.types.RenderEngine):
    bl_idname = 'nox_renderer'
    bl_label = 'NOX Renderer'
    bl_use_preview = False

    def render(self, scene):
        scale = scene.render.resolution_percentage / 100.0
        self.size_x = int(scene.render.resolution_x * scale)
        self.size_y = int(scene.render.resolution_y * scale)

        if scene.name == 'preview':
            self.render_preview(scene)
        else:
            #self.render_scene(scene)
            self.export_and_run_nox(scene)

    def export_and_run_nox(self, scene):
        noxpath = getNOXpath()
        if not noxpath[0]:
            return False
        timestr = time.strftime("_%Y_%m_%d_%H_%M_%S")
        noxscenefile = noxpath[1]+'\\exported scenes\\'+scene.name+timestr+".nox"
        print("exporting to file: "+noxscenefile)
        #bpy.ops.export.nox_data(filepath=noxscenefile)
        write_NOX_data(bpy.context, noxscenefile, 'OPT_A')
        print("starting nox")
        runNOX(noxscenefile, True)

    # In this example, we fill the preview renders with a flat green color.    
    def render_preview(self, scene):
        #print("------------------------")
        #print("RENDER PREVIEW")
        #print(dir(self))
        #for o in scene.objects:
        #    print(o.name + "           - "+o.type)
        #print("------------------------")

        pixel_count = self.size_x * self.size_y

        # The framebuffer is defined as a list of pixels, each pixel
        # itself being a list of R,G,B,A values
        green_rect = [[0.0, 1.0, 0.0, 1.0]] * pixel_count

        # Here we write the pixel values to the RenderResult
        result = self.begin_result(0, 0, self.size_x, self.size_y)
        layer = result.layers[0]
        layer.rect = green_rect
        self.end_result(result)
        pass

    # In this example, we fill the full renders with a flat blue color.
    def render_scene(self, scene):
        pixel_count = self.size_x * self.size_y

        # The framebuffer is defined as a list of pixels, each pixel
        # itself being a list of R,G,B,A values
        blue_rect = [[0.0, 0.0, 1.0, 1.0]] * pixel_count

        # Here we write the pixel values to the RenderResult
        result = self.begin_result(0, 0, self.size_x, self.size_y)
        layer = result.layers[0]
        layer.rect = blue_rect
        self.end_result(result)


# RenderEngines also need to tell UI Panels that they are compatible
# Otherwise most of the UI will be empty when the engine is selected.
# In this example, we need to see the main render image button and
# the material preview panel.

#import properties_render
#properties_render.RENDER_PT_render.COMPAT_ENGINES.add('nox_renderer')
#del properties_render

#import properties_material
#properties_material.MATERIAL_PT_preview.COMPAT_ENGINES.add('nox_renderer')
#del properties_material


#try:
#    #bpy.types.NOXMaterialProps.COMPAT_ENGINES.add('nox_renderer')
#    #bpy.types.MATERIAL_PT_context_material.COMPAT_ENGINES.add("nox_renderer")    
#	pass
#except:
#    pass
    